<?php
/**
 * MRWP_Orders_List_Table class
 *
 * @package mondialrelay
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

// WP_List_Table class inclusion
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class MRWP_Mass_List_Table extends WP_List_Table {

	/**
	 * The order modes for filtering
	 *
	 * @var array
	 */
	private $order_modes = array();

	/**
	 * The order dates for filtering
	 *
	 * @var array
	 */
	private $order_dates = array();

	/**
	 * The order status we look at
	 *
	 * @var array
	 */
	//protected $watched_statuses = array( 'wc-processing', 'on-hold' );
	protected $watched_statuses = array();

    public function __construct() {
        parent::__construct(
			array(
	            'singular' => 'expedition',     //singular name of the listed records
	            'plural'   => 'expeditions',    //plural name of the listed records
	            'ajax'     => true,             //ajax support
				'screen'   => 'mondial-relay_page_wp-mondialrelay-mass-expeditions',
			)
		);

		// All WooCommerce status with custom ones and without completed, cancelled, refunded, failed, draft
		$this->watched_statuses = array_diff(array_keys( wc_get_order_statuses() ), ["wc-completed", "wc-cancelled", "wc-refunded", "wc-failed", "wc-checkout-draft"]);
		//var_dump($this->watched_statuses);
    }

	/**
	 * @return bool
	 */
	public function ajax_user_can() {
		return current_user_can( 'shop_manager' );
	}

	public function prepare_items() {

		// column_headers
		$columns  = $this->get_columns();
		$hidden   = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array( $columns, $hidden, $sortable );

		$args = array(
			'limit'        => -1, // Query all orders
			'return'       => 'ids',
			'meta_key'     => 'Mondial Relay Shipping Code',
		);

		// If we have a valid status filter, use it, otherwise accept all statuses
		if ( isset( $_GET['filter_etat'] ) && in_array( $_GET['filter_etat'], $this->watched_statuses ) ) {
			$args['post_status'] = array( $_GET['filter_etat'] );
		} else {
			$args['post_status'] = $this->watched_statuses;
		}

		// All MR order IDs
		$default_modes = MRWP_modes::all_modes();

		// If we have a valid mode filter, use it, otherwise accept all MR modes
		if ( isset( $_GET['filter_mode'] ) && in_array( $_GET['filter_mode'], $default_modes ) ) {
			$args['meta_value'] = array( $_GET['filter_mode'] );
		} else {
			$args['meta_value'] = $default_modes;
		}

		// If we have a date filter, add it to the query args
		if ( isset( $_GET['filter_date'] ) ) {
			$args['date_created'] = $_GET['filter_date'];
		}

		// @see https://github.com/woocommerce/woocommerce/wiki/wc_get_orders-and-WC_Order_Query
		$all_order_ids = wc_get_orders( $args );

		$data = array();
		if ( count( $all_order_ids ) > 0 ) {
			// Get MR orders without expeditions
			$pending_orders = wc_get_orders(
				array(
					'limit'        => 200, // Max 200 orders
					'post__in'     => $all_order_ids,
					'orderby'      => 'date',
					'order'        => 'DESC',
					'meta_key'     => 'ExpeditionNum', // Our postmeta key
					'meta_compare' => 'NOT EXISTS',    // The comparison argument
				)
			);

			// For each order, the table is filled in
			foreach ( $pending_orders as $value ) {
				$OrderId     = $value->get_id();
				$OrderDate   = $value->get_date_created()->format ('Y-m-d');
				$OrderUser   = $value->get_formatted_shipping_full_name();
				$OrderAdress = $value->get_formatted_shipping_address();
				$OrderCode   = $value->get_meta('Mondial Relay Shipping Code');
				$OrderPoids  = $value->get_meta('Mondial Relay Parcel Weight');

				$data[] = array(
					'Num_Dossier'    => $OrderId,
					'Status'         => $value->get_status(),
					'Mode_Livraison' => $OrderCode,
					'Date'           => $OrderDate,
					'Client'         => $OrderUser,
					'Adresse'        => $OrderAdress,
					'Poids'          => $OrderPoids,
				);
			}
			$total_items  = count( $data );

			// Sort the orders
			usort( $data, array( get_class(), 'sort_by_column' ) );
		}

		// Save list of orders
		$this->items = $data;

		// Compute list of dates and delivery modes for filtering
		$dates = array();
		$modes = array();
		foreach( $this->items as $order ) {
			$dates[ $order['Date'] ] = true;
			$modes[ $order['Mode_Livraison'] ] = true;
		}
		$this->order_dates = array_keys( $dates );
		$this->order_modes = array_keys( $modes );
	}

	/**
	 * Returns order of 2 items sorted by the optionally specified column in the request
	 *
	 * @return int Relative order -1, 0 or 1
	 */
	private static function sort_by_column( $a, $b ) {

		// Set reasonable default values
		$orderby = ( ! empty( $_REQUEST['orderby'] ) ) ? $_REQUEST['orderby'] : 'Num_Dossier';
		$order   = ( ! empty( $_REQUEST['order'] ) ) ? $_REQUEST['order'] : 'desc';

		// Compare both values as strings
		$result  = strnatcmp( $a[ $orderby ], $b[ $orderby ] );

		// Reverse order sign if not sorting by asc
		return ( 'asc' === $order ) ? $result : -$result;
	}

	/**
	 * This function is used by the display code. If false, « No items found. » is displayed
	 *
	 * @return bool
	 */
	public function has_items() {
		return count( $this->items );
	}

	// Columns
    public function get_columns() {
        $columns = array(
            'cb'             => '<input type="checkbox" />', //Render a checkbox instead of text
            'Num_Dossier'    => __( 'Order', 'woocommerce' ),
            'Status'         => __( 'Status', 'woocommerce' ),
            'Mode_Livraison' => __( 'Mode de Livraison', 'mondialrelay-wordpress'),
            'Date'           => __( 'Date', 'woocommerce' ),
            'Client'         => __( 'Shipping', 'woocommerce' ),
            'Adresse'        => __( 'Ship to', 'woocommerce' ),
            'Poids'          => __( 'Poids', 'mondialrelay-wordpress'),
            'Expedition'     => __( 'Expédition', 'mondialrelay-wordpress'),
        );

        return $columns;
    }

	/**
	 * Returns list of sortable columns.
	 *
	 * @return array list of sortable columns
	 */
	function get_sortable_columns() {
		$sortable_columns = array(
			'Num_Dossier'    => array( 'Num_Dossier', 'asc' ),    // The second element sets the initial sorting order.
			'Mode_Livraison' => array( 'Mode_Livraison', 'asc' ), // Does not work if set as a string
			'Date'           => array( 'Date', 'asc' ),
			'Client'         => array( 'Client', 'asc' ),
		);

		return $sortable_columns;
	}

	/**
	 * Display our stuff before the standard table.
	 *
	 * Add various form fields and buttons above the table.
	 */
	public function display() {

		global $order_id; // Pas sûr que ça serve

		//Adds a nonce field
		wp_nonce_field( 'mrwp-list-nonce', 'mrwp_list_nonce' );
		?>
		<style>
			a.cancel-expedition {
				color: #ca4a1f;
			}
			a.dashicons-saved.save-weight {
				color: #24b47e;
			}
			.spinner-weight {
				animation: dashicons-spin 1s infinite;
				animation-timing-function: linear;
				color: #24b47e;
			}
			#labels {
				font-size: 1.3em;
				display: block;
				margin: 1em 0;
				font-weight: 600;
				color: #ca0047;
			}
			tr[data-status="success"] .Expedition {
				color: #24b47e;
				font-size: 1.3em;
				font-weight: 600;
			}

			tr[data-status="failure"] .Expedition {
				color: #ca0047;
				font-size: 1.3em;
				font-weight: 600;
			}

			@keyframes dashicons-spin {
			    0% {
					transform: rotate( 0deg );
			    }
			    100% {
				    transform: rotate( 360deg );
			    }
			}

		</style>
		<div style="text-align:center">
			<a href="#" id="create-expeditions" style="background-color: #24b47e !important; border-radius: 15px; border: none !important; box-shadow: none; color: #fff !important; min-height: 30px; padding-top: 2px; text-transform: uppercase; margin: 10px !important; text-align: center; text-shadow: none;" class="button save_order button-primary" data-order-id="<?php echo $order_id ?>">
				<?php _e( 'Création des expéditions en masse', 'mondialrelay-wordpress' ) ?>
			</a>
		</div>
		<div style="text-align:center">
			<label id="labels" style="display: none;"></label>
			<table style="margin:auto"><tr>
				<td><a class="button" id="ea4" style="display: none; background-color: #24b47e !important; border-radius: 15px; border: none !important; box-shadow: none; color: #fff !important; min-height: 30px; padding-top: 2px; text-transform: uppercase; margin: 5px !important; text-align: center; text-shadow: none;" target="_blank" href=""><?php _e( 'Étiquettes au format A4', 'mondialrelay-wordpress') ?></a>
				<a class="button" id="ea5" style="display: none; background-color: #24b47e !important; border-radius: 15px; border: none !important; box-shadow: none; color: #fff !important; min-height: 30px; padding-top: 2px; text-transform: uppercase; margin: 5px !important; text-align: center; text-shadow: none;" target="_blank" href=""><?php _e( 'Étiquettes au format A5', 'mondialrelay-wordpress') ?></a>
				<a class="button" id="ea10" style="display: none; background-color: #24b47e !important; border-radius: 15px; border: none !important; box-shadow: none; color: #fff !important; min-height: 30px; padding-top: 2px; text-transform: uppercase; margin: 5px !important; text-align: center; text-shadow: none;" target="_blank" href=""><?php _e( 'Étiquettes au format 10x15', 'mondialrelay-wordpress') ?></a></td>
			</tr></table>
		</div>
		<?php

		parent::display();
	}

	/**
	 * Extra controls to be displayed between bulk actions and pagination.
	 *
	 * @since 3.1.0
	 *
	 * @param string $which
	 */
	protected function extra_tablenav( $which ) {
		?>
		<div class="alignleft actions">
		<?php
		if ( 'top' === $which ) {
			ob_start();

			$selected_status = ! empty( $_GET['filter_etat'] ) ? $_GET['filter_etat'] : '';
			$selected_mode   = ! empty( $_GET['filter_mode'] ) ? $_GET['filter_mode'] : '';
			$selected_date   = ! empty( $_GET['filter_date'] ) ? $_GET['filter_date'] : '';
			?>
			<label for="filter-by-status" class="screen-reader-text"><?php _e( 'Filter by status' ); ?></label>
			<select name="filter_etat" id="filter-by-status">
				<option<?php selected( $selected_status, '' ); ?> value=""><?php _e( 'All statuses' ); ?></option>
				<?php
				foreach ( $this->watched_statuses as $status ) {
					printf(
						"<option %s value='%s'>%s</option>\n",
						selected( $selected_status, $status, false ),
						esc_attr( $status ),
						esc_html( wc_get_order_status_name( $status ) )
					);
				}
				?>
			</select>
			<label for="filter-by-mode" class="screen-reader-text"><?php _e( 'Filter by mode' ); ?></label>
			<select name="filter_mode" id="filter-by-mode">
				<option<?php selected( $selected_mode, '' ); ?> value=""><?php _e( 'All modes' ); ?></option>
				<?php
				foreach ( $this->order_modes as $mode ) {
					printf(
						"<option %s value='%s'>%s</option>\n",
						selected( $selected_mode, $mode, false ),
						esc_attr( $mode ),
						esc_html( $mode )
					);
				}
				?>
			</select>
			<label for="filter-by-date" class="screen-reader-text"><?php _e( 'Filter by date' ); ?></label>
			<select name="filter_date" id="filter-by-date">
				<option<?php selected( $selected_date, '' ); ?> value=""><?php _e( 'All dates' ); ?></option>
				<?php
				foreach ( $this->order_dates as $date ) {
					printf(
						"<option %s value='%s'>%s</option>\n",
						selected( $selected_date, $date, false ),
						esc_attr( $date ),
						esc_html( $date )
					);
				}
				?>
			</select>
			<?php

			$output = ob_get_clean();

			if ( ! empty( $output ) ) {
				echo $output;
				submit_button( __( 'Filter' ), '', 'filter_action', false, array( 'id' => 'filter-submit' ) );
			}
		}
		?>
		</div>
		<?php
	}

	/**
	 * Handles the column output.
	 *
	 * @param object $item The current item.
	 * @param string $column_name The current column name.
	 */
	protected function column_default( $item, $column_name ) {

		// Escape values
		$order_id = intval( $item['Num_Dossier'] );

        switch( $column_name ) {
            case 'Num_Dossier':
				return $order_id;

            case 'Status':
				return esc_html( wc_get_order_status_name( $item['Status'] ) );

            case 'Mode_Livraison':
				$html  = '<span class="mode-liv" data-id="' . $order_id . '">' . esc_html( $item['Mode_Livraison'] ) . '</span> ';
				$html .= '<a class="dashicons dashicons-no cancel-expedition" data-id="' . $order_id . '" title="Ignorer cette livraison"></a>';
				return $html;

            case 'Date':
            case 'Client':
                return esc_html( $item[ $column_name ] );

            case 'Adresse':
                return $item['Adresse'];

            case 'Poids':
				$html  = '<span class="weight" data-id="' . $order_id . '">' . esc_html( $item['Poids'] ) . '</span> ';
				$html .= '<a class="dashicons dashicons-edit update-weight" data-id="' . $order_id . '"></a>';
                return $html;

			case 'Expedition':
				return '';
           default:
                return print_r( $item, true ); //Show the whole array for troubleshooting purposes
        }
    }

	/**
	 * Handles the checkbox column output.
	 *
	 * @param array $item The current order.
	 */
	public function column_cb( $item ) {
		$order_id = intval( $item['Num_Dossier'] );
		?>
		<label class="screen-reader-text" for="cb-select-<?php echo $order_id; ?>">
			<?php
				/* translators: %s: Order ID. */
				printf( __( 'Sélectionner la commande n°%s' ), $order_id );
			?>
		</label>
		<input id="cb-select-<?php echo $order_id; ?>" type="checkbox" name="orders" value="<?php echo $order_id; ?>" />
		<?php
	}

	/**
	 * Handles an incoming ajax request (called from admin-ajax.php)
	 */
	public function ajax_response() {

		check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );

		$this->prepare_items();

		extract( $this->_args );
		extract( $this->_pagination_args, EXTR_SKIP );

		ob_start();
		if ( ! empty( $_REQUEST['no_placeholder'] ) ) {
			$this->display_rows();
		} else {
			$this->display_rows_or_placeholder();
		}
		$rows = ob_get_clean();

		ob_start();
		$this->print_column_headers();
		$headers = ob_get_clean();

		ob_start();
		$this->pagination('top');
		$pagination_top = ob_get_clean();

		ob_start();
		$this->pagination('bottom');
		$pagination_bottom = ob_get_clean();

		$response = array( 'rows' => $rows );
		$response['pagination']['top']    = $pagination_top;
		$response['pagination']['bottom'] = $pagination_bottom;
		$response['column_headers']       = $headers;

		if ( isset( $total_items ) ) {
			$response['total_items_i18n'] = sprintf( _n( '1 item', '%s items', $total_items ), number_format_i18n( $total_items ) );
        }

		if ( isset( $total_pages ) ) {
			$response['total_pages']      = $total_pages;
			$response['total_pages_i18n'] = number_format_i18n( $total_pages );
		}

		die( json_encode( $response ) );
	}

}
